@@ -1744,3 +1744,37 @@ addAccessibleTask(
   },
   { topLevel: false, iframe: true, remoteIframe: true }
 );
+
+/**
+ * Test keyboard focusable search predicate
+ */
+addAccessibleTask(
+  `
+  <p>Hello, <a href="http://www.example.com/" id="href">Example link</a> <a id="noHref">link without href</a></p>
+  <input id="input">
+  <button id="button">Click me</button>
+  <div id="container" style="height: 10px; overflow: auto;"><div style="height: 100px;"></div></div>
+  `,
+  async (browser, f, accDoc) => {
+    const searchPred = {
+      AXSearchKey: "AXKeyboardFocusableSearchKey",
+      AXResultsLimit: -1,
+      AXDirection: "AXDirectionNext",
+    };
+    const webArea = accDoc.nativeInterface.QueryInterface(
+      Ci.nsIAccessibleMacInterface
+    );
+
+    const results = webArea.getParameterizedAttributeValue(
+      "AXUIElementsForSearchPredicate",
+      NSDictionary(searchPred)
+    );
+
+    Assert.deepEqual(
+      results.map(r => r.getAttributeValue("AXDOMIdentifier")),
+      ["href", "input", "button"],
+      "Correct keyboard focusable search results"
+    );
+  },
+  { topLevel: false, iframe: true, remoteIframe: true }
+);
