@@ -45,6 +45,7 @@ export const MultiStageProtonScreen = props => {
       id={props.id}
       order={props.order}
       activeTheme={props.activeTheme}
+      installedAddons={props.installedAddons}
       screenMultiSelects={props.screenMultiSelects}
       setScreenMultiSelects={props.setScreenMultiSelects}
       activeMultiSelect={props.activeMultiSelect}
@@ -238,19 +239,19 @@ export class ProtonScreen extends React.PureComponent {
       <picture className={className} style={{ marginInline, marginBlock }}>
         {darkModeReducedMotionImageURL ? (
           <source
-            srcSet={darkModeReducedMotionImageURL}
+            srcset={darkModeReducedMotionImageURL}
             media="(prefers-color-scheme: dark) and (prefers-reduced-motion: reduce)"
           />
         ) : null}
         {darkModeImageURL ? (
           <source
-            srcSet={darkModeImageURL}
+            srcset={darkModeImageURL}
             media="(prefers-color-scheme: dark)"
           />
         ) : null}
         {reducedMotionImageURL ? (
           <source
-            srcSet={reducedMotionImageURL}
+            srcset={reducedMotionImageURL}
             media="(prefers-reduced-motion: reduce)"
           />
         ) : null}
@@ -278,6 +279,7 @@ export class ProtonScreen extends React.PureComponent {
         content.tiles.data ? (
           <AddonsPicker
             content={content}
+            installedAddons={this.props.installedAddons}
             message_id={this.props.messageId}
             handleAction={this.props.handleAction}
           />
@@ -360,13 +362,20 @@ export class ProtonScreen extends React.PureComponent {
   }
 
   renderStepsIndicator() {
-    const currentStep = (this.props.order ?? 0) + 1;
-    const previousStep = (this.props.previousOrder ?? -1) + 1;
-    const { content, totalNumberOfScreens: total } = this.props;
+    const {
+      order,
+      previousOrder,
+      content,
+      totalNumberOfScreens: total,
+      aboveButtonStepsIndicator,
+    } = this.props;
+    const currentStep = (order ?? 0) + 1;
+    const previousStep = (previousOrder ?? -1) + 1;
     return (
       <div
         id="steps"
         className={`steps${content.progress_bar ? " progress-bar" : ""}`}
+        above-button={aboveButtonStepsIndicator ? "" : null}
         data-l10n-id={
           content.steps_indicator?.string_id ||
           "onboarding-welcome-steps-indicator-label"
@@ -388,10 +397,7 @@ export class ProtonScreen extends React.PureComponent {
             totalNumberOfScreens={total}
           />
         ) : (
-          <StepsIndicator
-            order={this.props.order}
-            totalNumberOfScreens={total}
-          />
+          <StepsIndicator order={order} totalNumberOfScreens={total} />
         )}
       </div>
     );
@@ -413,6 +419,9 @@ export class ProtonScreen extends React.PureComponent {
             : {}
         }
       >
+        {content.dismiss_button && content.reverse_split
+          ? this.renderDismissButton()
+          : null}
         <Localized text={content.image_alt_text}>
           <div className="sr-only image-alt" role="img" />
         </Localized>
@@ -504,6 +513,7 @@ export class ProtonScreen extends React.PureComponent {
         className={`screen ${this.props.id || ""}
           ${screenClassName} ${textColorClass}`}
         reverse-split={content.reverse_split ? "" : null}
+        fullscreen={content.fullscreen ? "" : null}
         role={ariaRole ?? "alertdialog"}
         layout={content.layout}
         pos={content.position || "center"}
@@ -534,7 +544,9 @@ export class ProtonScreen extends React.PureComponent {
             />
           ) : null}
           {includeNoodles ? this.renderNoodles() : null}
-          {content.dismiss_button ? this.renderDismissButton() : null}
+          {content.dismiss_button && !content.reverse_split
+            ? this.renderDismissButton()
+            : null}
           <div
             className={`main-content ${hideStepsIndicator ? "no-steps" : ""}`}
             style={{
@@ -554,7 +566,9 @@ export class ProtonScreen extends React.PureComponent {
                 : null,
             }}
           >
-            {content.logo ? this.renderPicture(content.logo) : null}
+            {content.logo && !content.fullscreen
+              ? this.renderPicture(content.logo)
+              : null}
 
             {isRtamo ? (
               <div className="rtamo-icon">
@@ -576,6 +590,9 @@ export class ProtonScreen extends React.PureComponent {
                 justifyContent: content.split_content_justify_content,
               }}
             >
+              {content.logo && content.fullscreen
+                ? this.renderPicture(content.logo)
+                : null}
               {content.title || content.subtitle ? (
                 <div className={`welcome-text ${content.title_style || ""}`}>
                   {content.title ? this.renderTitle(content) : null}
