@@ -334,46 +334,46 @@ add_task(async function test_aboutwelcome_dismiss_button() {
 /**
  * Test rendering a screen with the "split" position
  */
-// add_task(async function test_aboutwelcome_split_position() {
-//   const TEST_SPLIT_STEP = makeTestContent("TEST_SPLIT_STEP", {
-//     position: "split",
-//     hero_text: "hero test",
-//   });
+add_task(async function test_aboutwelcome_split_position() {
+  const TEST_SPLIT_STEP = makeTestContent("TEST_SPLIT_STEP", {
+    position: "split",
+    hero_text: "hero test",
+  });
 
-//   const TEST_SPLIT_JSON = JSON.stringify([TEST_SPLIT_STEP]);
-//   let browser = await openAboutWelcome(TEST_SPLIT_JSON);
+  const TEST_SPLIT_JSON = JSON.stringify([TEST_SPLIT_STEP]);
+  let browser = await openAboutWelcome(TEST_SPLIT_JSON);
 
-//   await test_screen_content(
-//     browser,
-//     "renders screen secondary section containing hero text",
-//     // Expected selectors:
-//     [`main.screen[pos="split"]`, `.section-secondary`, `.message-text h1`]
-//   );
+  await test_screen_content(
+    browser,
+    "renders screen secondary section containing hero text",
+    // Expected selectors:
+    [`main.screen[pos="split"]`, `.section-secondary`, `.message-text h1`]
+  );
 
-//   // Ensure secondary section has split template styling
-//   await test_element_styles(
-//     browser,
-//     "main.screen .section-secondary",
-//     // Expected styles:
-//     {
-//       display: "flex",
-//       margin: "auto 0px auto auto",
-//     }
-//   );
+  // Ensure secondary section has split template styling
+  await test_element_styles(
+    browser,
+    "main.screen .section-secondary",
+    // Expected styles:
+    {
+      display: "flex",
+      margin: "auto 0px auto auto",
+    }
+  );
 
-//   // Ensure secondary action has button styling
-//   await test_element_styles(
-//     browser,
-//     ".action-buttons .secondary-cta .secondary",
-//     // Expected styles:
-//     {
-//       // Override default text-link styles
-//       "background-color": "color(srgb 0.0823529 0.0784314 0.101961 / 0.07)",
-//       color: "rgb(21, 20, 26)",
-//     }
-//   );
-//   browser.closeBrowser();
-// });
+  // Ensure secondary action has button styling
+  await test_element_styles(
+    browser,
+    ".action-buttons .secondary-cta .secondary",
+    // Expected styles:
+    {
+      // Override default text-link styles
+      "background-color": "color(srgb 0.0823529 0.0784314 0.101961 / 0.07)",
+      color: "rgb(21, 20, 26)",
+    }
+  );
+  browser.closeBrowser();
+});
 
 /**
  * Test rendering a screen with a URL value and default color for backdrop
@@ -399,7 +399,7 @@ add_task(async function test_aboutwelcome_with_url_backdrop() {
     // Expected selectors:
     [`div.outer-wrapper.onboardingContainer[style*='${TEST_BACKDROP_URL}']`]
   );
-  await doExperimentCleanup();
+  doExperimentCleanup();
   browser.closeBrowser();
 });
 
@@ -428,7 +428,7 @@ add_task(async function test_aboutwelcome_with_color_backdrop() {
     // Expected selectors:
     [`div.outer-wrapper.onboardingContainer[style*='${TEST_BACKDROP_COLOR}']`]
   );
-  await doExperimentCleanup();
+  doExperimentCleanup();
   browser.closeBrowser();
 });
 
@@ -492,7 +492,7 @@ add_task(async function test_aboutwelcome_with_text_color_override() {
     }
   );
 
-  await doExperimentCleanup();
+  doExperimentCleanup();
   await SpecialPowers.popPrefEnv();
   browser.closeBrowser();
 });
@@ -569,7 +569,7 @@ add_task(async function test_aboutwelcome_with_progress_bar() {
     );
   });
 
-  await doExperimentCleanup();
+  doExperimentCleanup();
   browser.closeBrowser();
 });
 
@@ -611,7 +611,7 @@ add_task(async function test_aboutwelcome_history_updates_disabled() {
     "No entries added to the session's history stack with history updates disabled"
   );
 
-  await doExperimentCleanup();
+  doExperimentCleanup();
   browser.closeBrowser();
 });
 
@@ -718,7 +718,7 @@ add_task(async function test_aboutwelcome_start_screen_configured() {
     ok(false, "No telemetry sent");
   }
 
-  await doExperimentCleanup();
+  doExperimentCleanup();
   browser.closeBrowser();
   sandbox.restore();
 });
@@ -743,6 +743,74 @@ add_task(async function test_aboutwelcome_rdm_property() {
     ["main.TEST_NO_RDM[no-rdm]"]
   );
 
-  await doExperimentCleanup();
+  doExperimentCleanup();
+  browser.closeBrowser();
+});
+
+/**
+ * Test rendering the dismiss button on a reversed split layout screen
+ */
+add_task(async function test_aboutwelcome_reverse_dismiss() {
+  let screens = [
+    makeTestContent(`TEST_REVERSE_DISMISS`, {
+      reverse_split: true,
+      position: "split",
+      dismiss_button: { action: { dismiss: true } },
+    }),
+  ];
+
+  let doExperimentCleanup = await ExperimentFakes.enrollWithFeatureConfig({
+    featureId: "aboutwelcome",
+    value: { enabled: true, screens },
+  });
+
+  let browser = await openAboutWelcome();
+
+  await test_screen_content(
+    browser,
+    "render screen with 'reverse_split' attribute",
+    // Expected selectors:
+    ["main.TEST_REVERSE_DISMISS[reverse-split]"]
+  );
+
+  await test_screen_content(
+    browser,
+    "renders screen with dismiss button on secondary section",
+    // Expected selectors:
+    [".section-secondary .dismiss-button"]
+  );
+
+  // Click dismiss button
+  await onButtonClick(browser, "button.dismiss-button");
+
+  // Wait for about:home to load
+  await BrowserTestUtils.browserLoaded(browser, false, "about:home");
+  is(browser.currentURI.spec, "about:home", "about:home loaded");
+
+  doExperimentCleanup();
+  browser.closeBrowser();
+});
+
+/**
+ * Test rendering a screen with that uses fullscreen mode
+ */
+add_task(async function test_aboutwelcome_fullscreen_property() {
+  let screens = [makeTestContent(`TEST_FULLSCREEN`, { fullscreen: true })];
+
+  let doExperimentCleanup = await ExperimentFakes.enrollWithFeatureConfig({
+    featureId: "aboutwelcome",
+    value: { enabled: true, screens },
+  });
+
+  let browser = await openAboutWelcome();
+
+  await test_screen_content(
+    browser,
+    "render screen with 'fullscreen' attribute",
+    // Expected selectors:
+    ["main.TEST_FULLSCREEN[fullscreen]"]
+  );
+
+  doExperimentCleanup();
   browser.closeBrowser();
 });
