@@ -319,6 +319,12 @@ describe("ActivityStream", () => {
         "getVariable"
       );
       sandbox.stub(global.Region, "home").get(() => "CA");
+      sandbox
+        .stub(global.Services.locale, "appLocaleAsBCP47")
+        .get(() => "en-CA");
+      getVariableStub
+        .withArgs("localeWeatherConfig")
+        .returns("en-US,en-CA,en-GB");
     });
     it("should turn off weather system pref if no region weather config is set and no geo is set", () => {
       getVariableStub.withArgs("regionWeatherConfig").returns("");
@@ -343,6 +349,44 @@ describe("ActivityStream", () => {
       assert.isFalse(PREFS_CONFIG.get("system.showWeather").value);
     });
   });
+  describe("discoverystream.locale-weather-config", () => {
+    let getVariableStub;
+    beforeEach(() => {
+      getVariableStub = sandbox.stub(
+        global.NimbusFeatures.pocketNewtab,
+        "getVariable"
+      );
+      sandbox.stub(global.Region, "home").get(() => "CA");
+      sandbox
+        .stub(global.Services.locale, "appLocaleAsBCP47")
+        .get(() => "en-CA");
+      getVariableStub.withArgs("regionWeatherConfig").returns("CA");
+    });
+    it("should turn off weather system pref if no locale weather config is set and no locale is set", () => {
+      getVariableStub.withArgs("localeWeatherConfig").returns("");
+      sandbox.stub(global.Services.locale, "appLocaleAsBCP47").get(() => "");
+
+      as._updateDynamicPrefs();
+
+      assert.isFalse(PREFS_CONFIG.get("system.showWeather").value);
+    });
+    it("should turn on weather system pref based on locale weather config pref", () => {
+      getVariableStub
+        .withArgs("localeWeatherConfig")
+        .returns("en-US,en-CA,en-GB");
+
+      as._updateDynamicPrefs();
+
+      assert.isTrue(PREFS_CONFIG.get("system.showWeather").value);
+    });
+    it("should turn off weather system pref if no locale weather config is set", () => {
+      getVariableStub.withArgs("localeWeatherConfig").returns("");
+
+      as._updateDynamicPrefs();
+
+      assert.isFalse(PREFS_CONFIG.get("system.showWeather").value);
+    });
+  });
   describe("_updateDynamicPrefs topstories default value", () => {
     let getVariableStub;
     let getBoolPrefStub;
