@@ -306,52 +306,14 @@
     }
 
     handleSearchCommand(aEvent, aEngine, aForceNewTab) {
-      let where = "current";
-      let params;
-      const newTabPref = Services.prefs.getBoolPref("browser.search.openintab");
-
-      // Open ctrl/cmd clicks on one-off buttons in a new background tab.
       if (
         aEvent &&
-        aEvent.originalTarget.classList.contains("search-go-button")
+        aEvent.originalTarget.classList.contains("search-go-button") &&
+        aEvent.button == 2
       ) {
-        if (aEvent.button == 2) {
-          return;
-        }
-        where = lazy.BrowserUtils.whereToOpenLink(aEvent, false, true);
-        if (
-          newTabPref &&
-          !aEvent.altKey &&
-          !aEvent.getModifierState("AltGraph") &&
-          where == "current" &&
-          !gBrowser.selectedTab.isEmpty
-        ) {
-          where = "tab";
-        }
-      } else if (aForceNewTab) {
-        where = "tab";
-        if (Services.prefs.getBoolPref("browser.tabs.loadInBackground")) {
-          where += "-background";
-        }
-      } else {
-        if (
-          (KeyboardEvent.isInstance(aEvent) &&
-            (aEvent.altKey || aEvent.getModifierState("AltGraph"))) ^
-            newTabPref &&
-          !gBrowser.selectedTab.isEmpty
-        ) {
-          where = "tab";
-        }
-        if (
-          MouseEvent.isInstance(aEvent) &&
-          (aEvent.button == 1 || aEvent.getModifierState("Accel"))
-        ) {
-          where = "tab";
-          params = {
-            inBackground: true,
-          };
-        }
+        return;
       }
+      let { where, params } = this._whereToOpen(aEvent, aForceNewTab);
       this.handleSearchCommandWhere(aEvent, aEngine, where, params);
     }
 
@@ -449,6 +411,97 @@
       openTrustedLinkIn(submission.uri.spec, aWhere, params);
     }
 
+    /**
+     * Returns information on where a search results page should be loaded: in the
+     * current tab or a new tab.
+     *
+     * @param {event} aEvent
+     *        The event that triggered the page load.
+     * @param {boolean} [aForceNewTab]
+     *        True to force the load in a new tab.
+     * @returns {object} An object { where, params }.  `where` is a string:
+     *          "current" or "tab".  `params` is an object further describing how
+     *          the page should be loaded.
+     */
+    _whereToOpen(aEvent, aForceNewTab = false) {
+      let where = "current";
+      let params = {};
+      const newTabPref = Services.prefs.getBoolPref("browser.search.openintab");
+
+      // Open ctrl/cmd clicks on one-off buttons in a new background tab.
+      if (aEvent?.originalTarget.classList.contains("search-go-button")) {
+        where = lazy.BrowserUtils.whereToOpenLink(aEvent, false, true);
+        if (
+          newTabPref &&
+          !aEvent.altKey &&
+          !aEvent.getModifierState("AltGraph") &&
+          where == "current" &&
+          !gBrowser.selectedTab.isEmpty
+        ) {
+          where = "tab";
+        }
+      } else if (aForceNewTab) {
+        where = "tab";
+        if (Services.prefs.getBoolPref("browser.tabs.loadInBackground")) {
+          params = {
+            inBackground: true,
+          };
+        }
+      } else {
+        if (
+          (KeyboardEvent.isInstance(aEvent) &&
+            (aEvent.altKey || aEvent.getModifierState("AltGraph"))) ^
+            newTabPref &&
+          !gBrowser.selectedTab.isEmpty
+        ) {
+          where = "tab";
+        }
+        if (
+          MouseEvent.isInstance(aEvent) &&
+          (aEvent.button == 1 || aEvent.getModifierState("Accel"))
+        ) {
+          where = "tab";
+          params = {
+            inBackground: true,
+          };
+        }
+      }
+
+      return { where, params };
+    }
+
+    /**
+     * Opens the search form of the provided engine or the current engine
+     * if no engine was provided.
+     *
+     * @param {event} aEvent
+     *        The event causing the searchForm to be opened.
+     * @param {nsISearchEngine} [aEngine]
+     *        The search engine or undefined to use the current engine.
+     * @param {string} where
+     *        Where the search form should be opened.
+     * @param {object} [params]
+     *        Parameters for URILoadingHelper.openLinkIn.
+     */
+    openSearchFormWhere(aEvent, aEngine, where, params = {}) {
+      let engine = aEngine || this.currentEngine;
+      let searchForm = engine.wrappedJSObject.searchForm;
+
+      if (where === "tab" && !!params.inBackground) {
+        // Keep the focus in the search bar.
+        params.avoidBrowserFocus = true;
+      } else if (
+        where !== "window" &&
+        aEvent.keyCode === KeyEvent.DOM_VK_RETURN
+      ) {
+        // Move the focus to the selected browser when keyup the Enter.
+        params.avoidBrowserFocus = true;
+        this._needBrowserFocusAtEnterKeyUp = true;
+      }
+
+      openTrustedLinkIn(searchForm, where, params);
+    }
+
     disconnectedCallback() {
       this.destroy();
       while (this.firstChild) {
@@ -734,7 +787,7 @@
         // Entering customization mode after the search bar had focus causes
         // the popup to appear again, due to focus returning after the
         // hamburger panel closes. Don't open in that spurious event.
-        if (document.documentElement.getAttribute("customizing") == "true") {
+        if (document.documentElement.hasAttribute("customizing")) {
           return;
         }
 
@@ -795,6 +848,25 @@
           this.textbox.selectedButton.open = !this.textbox.selectedButton.open;
           return true;
         }
+        // Ignore blank search unless add search engine or
+        // settings button is selected, see bugs 1894910 and 1903608.
+        if (
+          !this.textbox.value &&
+          !(
+            this.textbox.selectedButton?.getAttribute("id") ==
+              "searchbar-anon-search-settings" ||
+            this.textbox.selectedButton?.classList.contains(
+              "searchbar-engine-one-off-add-engine"
+            )
+          )
+        ) {
+          if (event.shiftKey) {
+            let engine = this.textbox.selectedButton?.engine;
+            let { where, params } = this._whereToOpen(event);
+            this.openSearchFormWhere(event, engine, where, params);
+          }
+          return true;
+        }
         // Otherwise, "call super": do what the autocomplete binding's
         // handleEnter implementation does.
         return this.textbox.mController.handleEnter(false, event || null);
