@@ -8,11 +8,11 @@
 #include "nsCSSPropertyID.h"
 #include "nsIFrame.h"
 #include "nsContainerFrame.h"
-#include "nsIScrollableFrame.h"
 #include "nsContentUtils.h"
 #include "nsLayoutUtils.h"
 #include "nsRefreshDriver.h"
 #include "mozilla/PresShell.h"
+#include "mozilla/ScrollContainerFrame.h"
 #include "mozilla/StaticPrefs_dom.h"
 #include "mozilla/StaticPrefs_layout.h"
 #include "mozilla/ServoBindings.h"
@@ -149,7 +149,7 @@ static void LazyLoadCallback(
     Element* target = entry->Target();
     if (entry->IsIntersecting()) {
       if (auto* image = HTMLImageElement::FromNode(target)) {
-        image->StopLazyLoading(HTMLImageElement::StartLoading::Yes);
+        image->StopLazyLoading();
       } else if (auto* iframe = HTMLIFrameElement::FromNode(target)) {
         iframe->StopLazyLoading();
       } else {
@@ -206,13 +206,24 @@ void DOMIntersectionObserver::GetThresholds(nsTArray<double>& aRetVal) {
 }
 
 void DOMIntersectionObserver::Observe(Element& aTarget) {
-  if (!mObservationTargetSet.EnsureInserted(&aTarget)) {
+  bool wasPresent =
+      mObservationTargetMap.WithEntryHandle(&aTarget, [](auto handle) {
+        if (handle.HasEntry()) {
+          return true;
+        }
+        handle.Insert(Uninitialized);
+        return false;
+      });
+  if (wasPresent) {
     return;
   }
-  aTarget.RegisterIntersectionObserver(this);
+  aTarget.BindObject(this, [](nsISupports* aObserver, nsINode* aTarget) {
+    static_cast<DOMIntersectionObserver*>(aObserver)->UnlinkTarget(
+        *aTarget->AsElement());
+  });
   mObservationTargets.AppendElement(&aTarget);
 
-  MOZ_ASSERT(mObservationTargets.Length() == mObservationTargetSet.Count());
+  MOZ_ASSERT(mObservationTargets.Length() == mObservationTargetMap.Count());
 
   Connect();
   if (mDocument) {
@@ -223,14 +234,14 @@ void DOMIntersectionObserver::Observe(Element& aTarget) {
 }
 
 void DOMIntersectionObserver::Unobserve(Element& aTarget) {
-  if (!mObservationTargetSet.EnsureRemoved(&aTarget)) {
+  if (!mObservationTargetMap.Remove(&aTarget)) {
     return;
   }
 
   mObservationTargets.RemoveElement(&aTarget);
-  aTarget.UnregisterIntersectionObserver(this);
+  aTarget.UnbindObject(this);
 
-  MOZ_ASSERT(mObservationTargets.Length() == mObservationTargetSet.Count());
+  MOZ_ASSERT(mObservationTargets.Length() == mObservationTargetMap.Count());
 
   if (mObservationTargets.IsEmpty()) {
     Disconnect();
@@ -239,7 +250,7 @@ void DOMIntersectionObserver::Unobserve(Element& aTarget) {
 
 void DOMIntersectionObserver::UnlinkTarget(Element& aTarget) {
   mObservationTargets.RemoveElement(&aTarget);
-  mObservationTargetSet.Remove(&aTarget);
+  mObservationTargetMap.Remove(&aTarget);
   if (mObservationTargets.IsEmpty()) {
     Disconnect();
   }
@@ -252,7 +263,7 @@ void DOMIntersectionObserver::Connect() {
 
   mConnected = true;
   if (mDocument) {
-    mDocument->AddIntersectionObserver(this);
+    mDocument->AddIntersectionObserver(*this);
   }
 }
 
@@ -263,12 +274,12 @@ void DOMIntersectionObserver::Disconnect() {
 
   mConnected = false;
   for (Element* target : mObservationTargets) {
-    target->UnregisterIntersectionObserver(this);
+    target->UnbindObject(this);
   }
   mObservationTargets.Clear();
-  mObservationTargetSet.Clear();
+  mObservationTargetMap.Clear();
   if (mDocument) {
-    mDocument->RemoveIntersectionObserver(this);
+    mDocument->RemoveIntersectionObserver(*this);
   }
 }
 
@@ -331,7 +342,7 @@ static Maybe<nsRect> ComputeTheIntersection(
       DOMIntersectionObserver::IsForProximityToViewport::Yes) {
     const auto& disp = *target->StyleDisplay();
     auto clipAxes = target->ShouldApplyOverflowClipping(&disp);
-    if (clipAxes != PhysicalAxes::None) {
+    if (!clipAxes.isEmpty()) {
       inflowRect = OverflowAreas::GetOverflowClipRect(
           inflowRect, inflowRect, clipAxes,
           target->OverflowClipMargin(clipAxes));
@@ -351,9 +362,10 @@ static Maybe<nsRect> ComputeTheIntersection(
       nsLayoutUtils::GetCrossDocParentFrameInProcess(target);
   while (containerFrame && containerFrame != aRoot) {
     // FIXME(emilio): What about other scroll frames that inherit from
-    // nsHTMLScrollFrame but have a different type, like nsListControlFrame?
+    // ScrollContainerFrame but have a different type, like nsListControlFrame?
     // This looks bogus in that case, but different bug.
-    if (nsIScrollableFrame* scrollFrame = do_QueryFrame(containerFrame)) {
+    if (ScrollContainerFrame* scrollContainerFrame =
+            do_QueryFrame(containerFrame)) {
       if (containerFrame->GetParent() == aRoot && !aRoot->GetParent()) {
         // This is subtle: if we're computing the intersection against the
         // viewport (the root frame), and this is its scroll frame, we really
@@ -361,7 +373,8 @@ static Maybe<nsRect> ComputeTheIntersection(
         // root margin, which is already in aRootBounds).
         break;
       }
-      nsRect subFrameRect = scrollFrame->GetScrollPortRect();
+      nsRect subFrameRect =
+          scrollContainerFrame->GetScrollPortRectAccountingForDynamicToolbar();
 
       // 3.1 Map intersectionRect to the coordinate space of container.
       nsRect intersectionRectRelativeToContainer =
@@ -384,7 +397,7 @@ static Maybe<nsRect> ComputeTheIntersection(
       const auto& disp = *containerFrame->StyleDisplay();
       auto clipAxes = containerFrame->ShouldApplyOverflowClipping(&disp);
       // 3.2 TODO: Apply clip-path.
-      if (clipAxes != PhysicalAxes::None) {
+      if (!clipAxes.isEmpty()) {
         // 3.1 Map intersectionRect to the coordinate space of container.
         const nsRect intersectionRectRelativeToContainer =
             nsLayoutUtils::TransformFrameRectToAncestor(
@@ -428,9 +441,9 @@ static Maybe<nsRect> ComputeTheIntersection(
   // the viewport already (but it's in the same document).
   nsRect rect = intersectionRect.value();
   if (aTarget->PresContext() != aRoot->PresContext()) {
-    if (nsIFrame* rootScrollFrame =
-            aTarget->PresShell()->GetRootScrollFrame()) {
-      nsLayoutUtils::TransformRect(aRoot, rootScrollFrame, rect);
+    if (nsIFrame* rootScrollContainerFrame =
+            aTarget->PresShell()->GetRootScrollContainerFrame()) {
+      nsLayoutUtils::TransformRect(aRoot, rootScrollContainerFrame, rect);
     }
   }
 
@@ -504,9 +517,10 @@ static Maybe<OopIframeMetrics> GetOopIframeMetrics(
   }
 
   nsRect inProcessRootRect;
-  if (nsIScrollableFrame* scrollFrame =
-          rootPresShell->GetRootScrollFrameAsScrollable()) {
-    inProcessRootRect = scrollFrame->GetScrollPortRect();
+  if (ScrollContainerFrame* rootScrollContainerFrame =
+          rootPresShell->GetRootScrollContainerFrame()) {
+    inProcessRootRect = rootScrollContainerFrame
+                            ->GetScrollPortRectAccountingForDynamicToolbar();
   }
 
   Maybe<LayoutDeviceRect> remoteDocumentVisibleRect =
@@ -543,10 +557,13 @@ IntersectionInput DOMIntersectionObserver::ComputeInput(
   if (aRoot && aRoot->IsElement()) {
     if ((rootFrame = aRoot->AsElement()->GetPrimaryFrame())) {
       nsRect rootRectRelativeToRootFrame;
-      if (nsIScrollableFrame* scrollFrame = do_QueryFrame(rootFrame)) {
+      if (ScrollContainerFrame* scrollContainerFrame =
+              do_QueryFrame(rootFrame)) {
         // rootRectRelativeToRootFrame should be the content rect of rootFrame,
         // not including the scrollbars.
-        rootRectRelativeToRootFrame = scrollFrame->GetScrollPortRect();
+        rootRectRelativeToRootFrame =
+            scrollContainerFrame
+                ->GetScrollPortRectAccountingForDynamicToolbar();
       } else {
         // rootRectRelativeToRootFrame should be the border rect of rootFrame.
         rootRectRelativeToRootFrame = rootFrame->GetRectRelativeToSelf();
@@ -576,11 +593,12 @@ IntersectionInput DOMIntersectionObserver::ComputeInput(
       // handle the OOP iframe positions.
       if (PresShell* presShell = rootDocument->GetPresShell()) {
         rootFrame = presShell->GetRootFrame();
-        // We use the root scrollable frame's scroll port to account the
+        // We use the root scroll container frame's scroll port to account the
         // scrollbars in rootRect, if needed.
-        if (nsIScrollableFrame* scrollFrame =
-                presShell->GetRootScrollFrameAsScrollable()) {
-          rootRect = scrollFrame->GetScrollPortRect();
+        if (ScrollContainerFrame* rootScrollContainerFrame =
+                presShell->GetRootScrollContainerFrame()) {
+          rootRect = rootScrollContainerFrame
+                         ->GetScrollPortRectAccountingForDynamicToolbar();
         } else if (rootFrame) {
           rootRect = rootFrame->GetRectRelativeToSelf();
         }
@@ -670,7 +688,7 @@ IntersectionOutput DOMIntersectionObserver::Intersect(
   if (aIsForProximityToViewport == IsForProximityToViewport::Yes) {
     const auto& disp = *targetFrame->StyleDisplay();
     auto clipAxes = targetFrame->ShouldApplyOverflowClipping(&disp);
-    if (clipAxes != PhysicalAxes::None) {
+    if (!clipAxes.isEmpty()) {
       targetRect = OverflowAreas::GetOverflowClipRect(
           targetRect, targetRect, clipAxes,
           targetFrame->OverflowClipMargin(clipAxes));
@@ -763,7 +781,15 @@ void DOMIntersectionObserver::Update(Document& aDocument,
     }
 
     // Steps 2.10 - 2.15.
-    if (target->UpdateIntersectionObservation(this, thresholdIndex)) {
+    bool updated = false;
+    if (auto entry = mObservationTargetMap.Lookup(target)) {
+      updated = entry.Data() != thresholdIndex;
+      entry.Data() = thresholdIndex;
+    } else {
+      MOZ_ASSERT_UNREACHABLE("Target not properly registered?");
+    }
+
+    if (updated) {
       // See https://github.com/w3c/IntersectionObserver/issues/432 about
       // why we use thresholdIndex > 0 rather than isIntersecting for the
       // entry's isIntersecting value.
