@@ -92,7 +92,6 @@ class nsIFrame;
 class nsIHTMLCollection;
 class nsIPrincipal;
 class nsIScreen;
-class nsIScrollableFrame;
 class nsIURI;
 class nsObjectLoadingContent;
 class nsPresContext;
@@ -110,6 +109,7 @@ class MappedDeclarationsBuilder;
 class EditorBase;
 class ErrorResult;
 class OOMReporter;
+class ScrollContainerFrame;
 class SMILAttr;
 struct MutationClosureData;
 class TextEditor;
@@ -120,6 +120,7 @@ namespace dom {
 struct CheckVisibilityOptions;
 struct CustomElementData;
 struct SetHTMLOptions;
+struct GetHTMLOptions;
 struct GetAnimationsOptions;
 struct ScrollIntoViewOptions;
 struct ScrollToOptions;
@@ -143,8 +144,6 @@ template <typename T>
 class Optional;
 enum class CallerType : uint32_t;
 enum class ReferrerPolicy : uint8_t;
-typedef nsTHashMap<nsRefPtrHashKey<DOMIntersectionObserver>, int32_t>
-    IntersectionObserverList;
 }  // namespace dom
 }  // namespace mozilla
 
@@ -194,8 +193,12 @@ enum : uint32_t {
   // algorithm https://w3c.github.io/webappsec-csp/#is-element-nonceable.
   ELEMENT_PARSER_HAD_DUPLICATE_ATTR_ERROR = ELEMENT_FLAG_BIT(6),
 
+  // If this flag is set, this element is in
+  // Document::mContentIdentifiersForLCP.
+  ELEMENT_IN_CONTENT_IDENTIFIER_FOR_LCP = ELEMENT_FLAG_BIT(7),
+
   // Remaining bits are for subclasses
-  ELEMENT_TYPE_SPECIFIC_BITS_OFFSET = NODE_TYPE_SPECIFIC_BITS_OFFSET + 7
+  ELEMENT_TYPE_SPECIFIC_BITS_OFFSET = NODE_TYPE_SPECIFIC_BITS_OFFSET + 8
 };
 
 #undef ELEMENT_FLAG_BIT
@@ -252,6 +255,21 @@ class Grid;
     ExplicitlySetAttrElement(nsGkAtoms::attr, aElement); \
   }
 
+// TODO(keithamus): Reference the spec link once merged.
+// https://github.com/whatwg/html/pull/9841/files#diff-41cf6794ba4200b839c53531555f0f3998df4cbb01a4d5cb0b94e3ca5e23947dR86024
+enum class InvokeAction : uint8_t {
+  Invalid,
+  Custom,
+  Auto,
+  TogglePopover,
+  ShowPopover,
+  HidePopover,
+  ShowModal,
+  Toggle,
+  Close,
+  Open,
+};
+
 class Element : public FragmentOrElement {
  public:
 #ifdef MOZILLA_INTERNAL_API
@@ -711,8 +729,8 @@ class Element : public FragmentOrElement {
 
  public:
   MOZ_CAN_RUN_SCRIPT
-  nsIScrollableFrame* GetScrollFrame(nsIFrame** aFrame = nullptr,
-                                     FlushType aFlushType = FlushType::Layout);
+  ScrollContainerFrame* GetScrollContainerFrame(
+      nsIFrame** aFrame = nullptr, FlushType aFlushType = FlushType::Layout);
 
  private:
   // Style state computed from element's state and style locks.
@@ -1112,8 +1130,23 @@ class Element : public FragmentOrElement {
     return FindAttributeDependence(aAttribute, aMaps, N);
   }
 
-  MOZ_CAN_RUN_SCRIPT virtual void HandleInvokeInternal(nsAtom* aAction,
-                                                       ErrorResult& aRv) {}
+  virtual bool IsValidInvokeAction(InvokeAction aAction) const {
+    return aAction == InvokeAction::Auto;
+  }
+
+  /**
+   * Elements can provide their own default behaviours for "Invoke" (see
+   * invoketarget/invokeaction attributes).
+   * If the action is not recognised, they can choose to ignore it and `return
+   * false`. If an action is recognised then they should `return true` to
+   * indicate to sub-classes that this has been handled and no further steps
+   * should be run.
+   */
+  MOZ_CAN_RUN_SCRIPT virtual bool HandleInvokeInternal(Element* invoker,
+                                                       InvokeAction aAction,
+                                                       ErrorResult& aRv) {
+    return false;
+  }
 
  private:
   void DescribeAttribute(uint32_t index, nsAString& aOutDescription) const;
@@ -1359,11 +1392,13 @@ class Element : public FragmentOrElement {
 
   enum class DelegatesFocus : bool { No, Yes };
   enum class ShadowRootClonable : bool { No, Yes };
+  enum class ShadowRootSerializable : bool { No, Yes };
 
   already_AddRefed<ShadowRoot> AttachShadowWithoutNameChecks(
       ShadowRootMode aMode, DelegatesFocus = DelegatesFocus::No,
       SlotAssignmentMode aSlotAssignmentMode = SlotAssignmentMode::Named,
-      ShadowRootClonable aClonable = ShadowRootClonable::No);
+      ShadowRootClonable aClonable = ShadowRootClonable::No,
+      ShadowRootSerializable aSerializable = ShadowRootSerializable::No);
 
   // Attach UA Shadow Root if it is not attached.
   enum class NotifyUAWidgetSetup : bool { No, Yes };
@@ -1543,6 +1578,7 @@ class Element : public FragmentOrElement {
 
   void SetHTML(const nsAString& aInnerHTML, const SetHTMLOptions& aOptions,
                ErrorResult& aError);
+  void GetHTML(const GetHTMLOptions& aOptions, nsAString& aResult);
 
   //----------------------------------------
 
@@ -1788,12 +1824,6 @@ class Element : public FragmentOrElement {
   // to it.
   void ClearDataset();
 
-  void RegisterIntersectionObserver(DOMIntersectionObserver* aObserver);
-  void UnregisterIntersectionObserver(DOMIntersectionObserver* aObserver);
-  void UnlinkIntersectionObservers();
-  bool UpdateIntersectionObservation(DOMIntersectionObserver* aObserver,
-                                     int32_t threshold);
-
   // A number of methods to cast to various XUL interfaces. They return a
   // pointer only if the element implements that interface.
   already_AddRefed<nsIDOMXULButtonElement> AsXULButton();
@@ -2100,17 +2130,29 @@ class Element : public FragmentOrElement {
    */
   virtual already_AddRefed<nsIURI> GetHrefURI() const { return nullptr; }
 
+  // Step 2. of
+  // <https://html.spec.whatwg.org/multipage/semantics.html#get-an-element's-target>
+  //
+  // Sanitize targets that look like they contain dangling markup.
+  static void SanitizeLinkOrFormTarget(nsAString& aTarget);
+
   /**
+   * <https://html.spec.whatwg.org/multipage/semantics.html#get-an-element's-target>
+   * (Excluding <form>)
+   *
    * Get the target of this link element. Consumers should established that
    * this element is a link (probably using IsLink) before calling this
-   * function (or else why call it?)
+   * function (or else why call it?). This method neuters probably markup
+   * injection attempts.
    *
    * Note: for HTML this gets the value of the 'target' attribute; for XLink
    * this gets the value of the xlink:_moz_target attribute, or failing that,
    * the value of xlink:show, converted to a suitably equivalent named target
    * (e.g. _blank).
    */
-  virtual void GetLinkTarget(nsAString& aTarget);
+  void GetLinkTarget(nsAString& aTarget);
+
+  virtual void GetLinkTargetImpl(nsAString& aTarget);
 
   virtual bool Translate() const;
 
