@@ -34,11 +34,19 @@ class DOMRectList;
 class InspectorFontFace;
 class Selection;
 
-enum class CollapsePolicy : uint8_t {
-  No,                                       // Don't need to collapse
-  DefaultRange,                             // Collapse the default range
-  DefaultRangeAndCrossShadowBoundaryRanges  // Collapse both the default range
-                                            // and the cross boundary range
+enum class RangeBehaviour : uint8_t {
+  // Keep both ranges
+  KeepDefaultRangeAndCrossShadowBoundaryRanges,
+  // Merge both ranges; This is the case where the range boundaries was in
+  // different roots initially, and becoming in the same roots now. Since
+  // they start to be in the same root, using normal range is good enough
+  // to represent it
+  MergeDefaultRangeAndCrossShadowBoundaryRanges,
+  // Collapse the default range
+  CollapseDefaultRange,
+  // Collapse both the default range and the cross-shadow-boundary range
+  CollapseDefaultRangeAndCrossShadowBoundaryRanges
+
 };
 }  // namespace dom
 }  // namespace mozilla
@@ -215,12 +223,15 @@ class nsRange final : public mozilla::dom::AbstractRange,
   void DeleteContents(ErrorResult& aRv);
   already_AddRefed<mozilla::dom::DocumentFragment> ExtractContents(
       ErrorResult& aErr);
-  nsINode* GetCommonAncestorContainer(ErrorResult& aRv) const {
+  nsINode* GetCommonAncestorContainer(
+      ErrorResult& aRv,
+      AllowRangeCrossShadowBoundary aAllowCrossShadowBoundary =
+          AllowRangeCrossShadowBoundary::No) const {
     if (!mIsPositioned) {
       aRv.Throw(NS_ERROR_NOT_INITIALIZED);
       return nullptr;
     }
-    return GetClosestCommonInclusiveAncestor();
+    return GetClosestCommonInclusiveAncestor(aAllowCrossShadowBoundary);
   }
   void InsertNode(nsINode& aNode, ErrorResult& aErr);
   bool IntersectsNode(nsINode& aNode, ErrorResult& aRv);
@@ -289,6 +300,11 @@ class nsRange final : public mozilla::dom::AbstractRange,
                                JS::Handle<JSObject*> aGivenProto) final;
   DocGroup* GetDocGroup() const;
 
+  // Given a CharacterDataChangeInfo and an RangeBoundary of where the
+  // character changes occurred at, compute the new boundary.
+  static RawRangeBoundary ComputeNewBoundaryWhenBoundaryInsideChangedText(
+      const CharacterDataChangeInfo& aInfo, const RawRangeBoundary& aBoundary);
+
  private:
   // no copy's or assigns
   nsRange(const nsRange&);
@@ -491,8 +507,8 @@ class nsRange final : public mozilla::dom::AbstractRange,
       const mozilla::RangeBoundaryBase<SPT, SRT>& aStartBoundary,
       const mozilla::RangeBoundaryBase<EPT, ERT>& aEndBoundary,
       nsINode* aRootNode, bool aNotInsertedYet = false,
-      mozilla::dom::CollapsePolicy aCollapsePolicy = mozilla::dom::
-          CollapsePolicy::DefaultRangeAndCrossShadowBoundaryRanges);
+      mozilla::dom::RangeBehaviour aRangeBehaviour = mozilla::dom::
+          RangeBehaviour::CollapseDefaultRangeAndCrossShadowBoundaryRanges);
 
   // Assume that this is guaranteed that this is held by the caller when
   // this is used.  (Note that we cannot use AutoRestore for mCalledByJS
